@@ -11,7 +11,7 @@ function run_install_test() {
   local PYTHON=$2
   local PYPI=$3
 
-  virtualenv --no-site-packages -p `which $PYTHON` test-venv
+  virtualenv -p `which $PYTHON` test-venv
 
   # Intentionally put a broken protoc in the path to make sure installation
   # doesn't require protoc installed.
@@ -19,11 +19,24 @@ function run_install_test() {
   chmod +x test-venv/bin/protoc
 
   source test-venv/bin/activate
-  pip install -i ${PYPI} protobuf==${VERSION} --no-cache-dir
+  (pip install -i ${PYPI} protobuf==${VERSION} --no-cache-dir) || (retry_pip_install ${PYPI} ${VERSION})
   deactivate
   rm -fr test-venv
 }
 
+function retry_pip_install() {
+  local PYPI=$1
+  local VERSION=$2
+  
+  read -p "pip install failed, possibly due to delay between upload and availability on pip. Retry? [y/n]" -r
+  echo
+  if [[ ! $REPLY =~ ^[Yy]$ ]]; then
+    exit 1
+  fi
+
+  (pip install -i ${PYPI} protobuf==${VERSION} --no-cache-dir) || (retry_pip_install ${PYPI} ${VERSION})
+}
+
 
 [ $# -lt 1 ] && {
   echo "Usage: $0 VERSION ["
@@ -73,25 +86,29 @@ else
   sed -i -r "s/__version__ = '.*'/__version__ = '${VERSION}.${DEV}'/" python/google/protobuf/__init__.py
 fi
 
+# Copy LICENSE
+cp LICENSE python/LICENSE
+
 cd python
 
 # Run tests locally.
-python setup.py build
-python setup.py test
+python3 setup.py build
+python3 setup.py test
 
 # Deploy source package to testing PyPI
-python setup.py sdist
+python3 setup.py sdist
 twine upload --skip-existing -r testpypi -u protobuf-wheel-test dist/*
 
-# Test locally with different python versions.
-run_install_test ${TESTING_VERSION} python2.7 https://test.pypi.org/simple
+# Sleep to allow time for distribution to be available on pip.
+sleep 5m
+
+# Test locally.
 run_install_test ${TESTING_VERSION} python3 https://test.pypi.org/simple
 
 # Deploy egg/wheel packages to testing PyPI and test again.
-python setup.py clean build bdist_wheel
+python3 setup.py clean build bdist_wheel
 twine upload --skip-existing -r testpypi -u protobuf-wheel-test dist/*
-
-run_install_test ${TESTING_VERSION} python2.7 https://test.pypi.org/simple
+sleep 5m
 run_install_test ${TESTING_VERSION} python3 https://test.pypi.org/simple
 
 echo "All install tests have passed using testing PyPI."
@@ -105,14 +122,14 @@ if [ $TESTING_ONLY -eq 0 ]; then
   echo "Publishing to PyPI..."
   # Be sure to run build before sdist, because otherwise sdist will not include
   # well-known types.
-  python setup.py clean build sdist
+  python3 setup.py clean build sdist
   twine upload --skip-existing -u protobuf-packages dist/*
   # Be sure to run clean before bdist_xxx, because otherwise bdist_xxx will
   # include files you may not want in the package. E.g., if you have built
   # and tested with --cpp_implemenation, bdist_xxx will include the _message.so
   # file even when you no longer pass the --cpp_implemenation flag. See:
   #   https://github.com/protocolbuffers/protobuf/issues/3042
-  python setup.py clean build bdist_wheel
+  python3 setup.py clean build bdist_wheel
   twine upload --skip-existing -u protobuf-packages dist/*
 else
   # Set the version number back (i.e., remove dev suffix).
